// CQuadTree.cpp: implementation of the CQuadTree class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "QuadTree.h"
#include "LogFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CQuadTree::CQuadTree()
{
	AfxEnableMemoryTracking( FALSE );
//	LogFile->AddLog("unable create quad tree without data",CLogFile::modeFail);
}
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CQuadTree::CQuadTree( CmprLandscape* map )
{
	root = new CQtrNode;
	buildNode( root,0,0,map->Header.dwXSectorsCount*32+1,map->Header.dwYSectorsCount*32+1);
}
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CQuadTree::~CQuadTree()
{
	delete root;

}
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void CQuadTree::buildNode(CQtrNode* node, unsigned int uBase_x, unsigned int uBase_y, unsigned int uWidth, unsigned int uHeight)
{
//	FILE *f = fopen("c:\\dl.txt","a+");
//	fprintf(f,"%d,%d,%d,%d\n", uBase_x, uBase_y, uWidth, uHeight );
//	fclose(	f);
	unsigned int	i,j,lastTex;
	float min,max;
	WORD angle;
	
	i = uBase_x;
	j = uBase_y;
	min=max=map->HeightMap[ i+map->iMapSizeX*j ].usZ;
	if( (uWidth >= NODE_MIN_SIZE) && (uHeight >= NODE_MIN_SIZE) )
	{

		node->childs = new CQtrNode[4];
		buildNode( node->childs + 0, uBase_x           , uBase_y           , uWidth/2, uHeight/2);
		buildNode( node->childs + 1, uBase_x + uWidth/2, uBase_y           , uWidth/2, uHeight/2);
		buildNode( node->childs + 2, uBase_x + uWidth/2, uBase_y + uHeight/2, uWidth/2, uHeight/2);
		buildNode( node->childs + 3, uBase_x		   , uBase_y + uHeight/2, uWidth/2, uHeight/2);

		float _min[4],_max[4];
		_min[0] = node->childs[0].points[0].z;
		_min[1] = node->childs[1].points[0].z;
		_min[2] = node->childs[2].points[0].z;
		_min[3] = node->childs[3].points[0].z;
		_max[0] = node->childs[0].points[7].z;
		_max[1] = node->childs[1].points[7].z;
		_max[2] = node->childs[2].points[7].z;
		_max[3] = node->childs[3].points[7].z;
		min = min(min(_min[0],_min[1]),min(_min[2],_min[3]));
		max = max(max(_max[0],_max[1]),max(_max[2],_max[3]));
		node->setPoints( (float)uBase_x,(float)uBase_y,min,(float)uBase_x+uWidth,(float)uBase_y+uHeight,max );
	}
	else 
	{
		node->childs = NULL;
		node->uTerrainList = glGenLists( 1 );
		glNewList( node->uTerrainList, GL_COMPILE );
		lastTex = 0;
			for( i=uBase_x;i<uBase_x+uWidth ;i+=2 )
			for( j=uBase_y;j<uBase_y+uHeight;j+=2 )
			{
			//////////////////////////////////////////////////////////////////////////////////
			float m[3][3][2];
			m[0][0][0] = 0.0f;
			m[0][0][1] = 0.0f;
			m[1][0][0] = 0.5f;
			m[1][0][1] = 0.0f;
			m[2][0][0] = 1.0f;
			m[2][0][1] = 0.0f;
			m[0][1][0] = 0.0f;
			m[0][1][1] = 0.5f;
			m[1][1][0] = 0.5f;
			m[1][1][1] = 0.5f;
			m[2][1][0] = 1.0f;
			m[2][1][1] = 0.5f;
			m[0][2][0] = 0.0f;
			m[0][2][1] = 1.0f;
			m[1][2][0] = 0.5f;
			m[1][2][1] = 1.0f;
			m[2][2][0] = 1.0f;
			m[2][2][1] = 1.0f;
			//////////////////////////////////////////////////////////////////////////////////////
			float tmp[3][3][2];
			memcpy( tmp,m,4*3*3*2);
			angle = (map->wTiles[i/2+j/2*map->Header.dwXSectorsCount*16] >> 14); 
			if(angle == 1)
			{
				m[0][0][0] = tmp[2][0][0];
				m[0][0][1] = tmp[2][0][1];

				m[1][0][0] = tmp[2][1][0];
				m[1][0][1] = tmp[2][1][1];

				m[2][0][0] = tmp[2][2][0];
				m[2][0][1] = tmp[2][2][1];

				m[0][1][0] = tmp[1][0][0];
				m[0][1][1] = tmp[1][0][1];

				m[1][1][0] = tmp[1][1][0];
				m[1][1][1] = tmp[1][1][1];

				m[2][1][0] = tmp[1][2][0];
				m[2][1][1] = tmp[1][2][1];

				m[0][2][0] = tmp[0][0][0];
				m[0][2][1] = tmp[0][0][1];

				m[1][2][0] = tmp[0][1][0];
				m[1][2][1] = tmp[0][1][1];

				m[2][2][0] = tmp[0][2][0];
				m[2][2][1] = tmp[0][2][1];

			}
			if(angle == 2)
			{
				m[0][0][0] = tmp[2][2][0];
				m[0][0][1] = tmp[2][2][1];

				m[1][0][0] = tmp[1][2][0];
				m[1][0][1] = tmp[1][2][1];

				m[2][0][0] = tmp[0][2][0];
				m[2][0][1] = tmp[0][2][1];

				m[0][1][0] = tmp[2][1][0];
				m[0][1][1] = tmp[2][1][1];

				m[1][1][0] = tmp[1][1][0];
				m[1][1][1] = tmp[1][1][1];

				m[2][1][0] = tmp[0][1][0];
				m[2][1][1] = tmp[0][1][1];

				m[0][2][0] = tmp[2][0][0];
				m[0][2][1] = tmp[2][0][1];

				m[1][2][0] = tmp[1][0][0];
				m[1][2][1] = tmp[1][0][1];

				m[2][2][0] = tmp[0][0][0];
				m[2][2][1] = tmp[0][0][1];

			}
			if(angle == 3)
			{
				m[0][0][0] = tmp[0][2][0];
				m[0][0][1] = tmp[0][2][1];

				m[1][0][0] = tmp[0][1][0];
				m[1][0][1] = tmp[0][1][1];

				m[2][0][0] = tmp[0][0][0];
				m[2][0][1] = tmp[0][0][1];

				m[0][1][0] = tmp[1][2][0];
				m[0][1][1] = tmp[1][2][1];

				m[1][1][0] = tmp[1][1][0];
				m[1][1][1] = tmp[1][1][1];

				m[2][1][0] = tmp[1][0][0];
				m[2][1][1] = tmp[1][0][1];

				m[0][2][0] = tmp[2][2][0];
				m[0][2][1] = tmp[2][2][1];

				m[1][2][0] = tmp[2][1][0];
				m[1][2][1] = tmp[2][1][1];

				m[2][2][0] = tmp[2][0][0];
				m[2][2][1] = tmp[2][0][1];

			}

				if( lastTex !=  map->textures[map->wTiles[i/2+j/2*map->Header.dwXSectorsCount*16] & 0x3ff])
				{
					glBindTexture( GL_TEXTURE_2D, map->textures[map->wTiles[i/2+j/2*map->Header.dwXSectorsCount*16] & 0x3ff] );
					lastTex = map->textures[map->wTiles[i/2+j/2*map->Header.dwXSectorsCount*16] & 0x3ff];
				}
				glBegin(GL_QUADS);
				glTexCoord2f(m[0][0][0],m[0][0][1]);
				vertexToDL( i,j );
				glTexCoord2f(m[1][0][0],m[1][0][1]);
				vertexToDL( i+1,j );
				glTexCoord2f(m[1][1][0],m[1][1][1]);
				vertexToDL( i+1,j+1 );
				glTexCoord2f(m[0][1][0],m[0][1][1]);
				vertexToDL( i,j+1 );
				glTexCoord2f(m[1][0][0],m[1][0][1]);
				vertexToDL( i+1,j );
				glTexCoord2f(m[2][0][0],m[2][0][1]);
				vertexToDL( i+2,j );
				glTexCoord2f(m[2][1][0],m[2][1][1]);
				vertexToDL( i+2,j+1 );
				glTexCoord2f(m[1][1][0],m[1][1][1]);
				vertexToDL( i+1,j+1 );
				glTexCoord2f(m[0][1][0],m[0][1][1]);
				vertexToDL( i,j+1 );
				glTexCoord2f(m[1][1][0],m[1][1][1]);
				vertexToDL( i+1,j+1 );
				glTexCoord2f(m[1][2][0],m[1][2][1]);
				vertexToDL( i+1,j+2 );
				glTexCoord2f(m[0][2][0],m[0][2][1]);
				vertexToDL( i,j+2 );
				glTexCoord2f(m[1][1][0],m[1][1][1]);
				vertexToDL( i+1,j+1 );
				glTexCoord2f(m[2][1][0],m[2][1][1]);
				vertexToDL( i+2,j+1 );
				glTexCoord2f(m[2][2][0],m[2][2][1]);
				vertexToDL( i+2,j+2 );
				glTexCoord2f(m[1][2][0],m[1][2][1]);
				vertexToDL( i+1,j+2 );
				glEnd();
			}
		glEndList();

		node->uAnimTilesList = glGenLists(1);
		if( !glIsList( node->uAnimTilesList ) )
		{
			Log->AddLog("can`t generate display list",CLogFile::modeFail);
		}
		glNewList( node->uAnimTilesList,GL_COMPILE );
		lastTex = 0;
		for( i=uBase_x;i<uBase_x+uWidth ;i+=2 )
		for( j=uBase_y;j<uBase_y+uHeight;j+=2 )
		{
			if( map->wObjectID[i/2+j/2*map->Header.dwXSectorsCount*16] != 0xffff )
			{

			float m[3][3][2];
			m[0][0][0] = 0.0f;
			m[0][0][1] = 0.0f;
			m[1][0][0] = 0.5f;
			m[1][0][1] = 0.0f;
			m[2][0][0] = 1.0f;
			m[2][0][1] = 0.0f;
			m[0][1][0] = 0.0f;
			m[0][1][1] = 0.5f;
			m[1][1][0] = 0.5f;
			m[1][1][1] = 0.5f;
			m[2][1][0] = 1.0f;
			m[2][1][1] = 0.5f;
			m[0][2][0] = 0.0f;
			m[0][2][1] = 1.0f;
			m[1][2][0] = 0.5f;
			m[1][2][1] = 1.0f;
			m[2][2][0] = 1.0f;
			m[2][2][1] = 1.0f;
			//////////////////////////////////////////////////////////////////////////////////////
			float tmp[3][3][2];
			memcpy( tmp,m,4*3*3*2);
			angle = (map->wAnimTiles[i/2+j/2*map->Header.dwXSectorsCount*16] >> 14); 
			if(angle == 1)
			{
				m[0][0][0] = tmp[2][0][0];
				m[0][0][1] = tmp[2][0][1];

				m[1][0][0] = tmp[2][1][0];
				m[1][0][1] = tmp[2][1][1];

				m[2][0][0] = tmp[2][2][0];
				m[2][0][1] = tmp[2][2][1];

				m[0][1][0] = tmp[1][0][0];
				m[0][1][1] = tmp[1][0][1];

				m[1][1][0] = tmp[1][1][0];
				m[1][1][1] = tmp[1][1][1];

				m[2][1][0] = tmp[1][2][0];
				m[2][1][1] = tmp[1][2][1];

				m[0][2][0] = tmp[0][0][0];
				m[0][2][1] = tmp[0][0][1];

				m[1][2][0] = tmp[0][1][0];
				m[1][2][1] = tmp[0][1][1];

				m[2][2][0] = tmp[0][2][0];
				m[2][2][1] = tmp[0][2][1];

			}
			if(angle == 2)
			{
				m[0][0][0] = tmp[2][2][0];
				m[0][0][1] = tmp[2][2][1];

				m[1][0][0] = tmp[1][2][0];
				m[1][0][1] = tmp[1][2][1];

				m[2][0][0] = tmp[0][2][0];
				m[2][0][1] = tmp[0][2][1];

				m[0][1][0] = tmp[2][1][0];
				m[0][1][1] = tmp[2][1][1];

				m[1][1][0] = tmp[1][1][0];
				m[1][1][1] = tmp[1][1][1];

				m[2][1][0] = tmp[0][1][0];
				m[2][1][1] = tmp[0][1][1];

				m[0][2][0] = tmp[2][0][0];
				m[0][2][1] = tmp[2][0][1];

				m[1][2][0] = tmp[1][0][0];
				m[1][2][1] = tmp[1][0][1];

				m[2][2][0] = tmp[0][0][0];
				m[2][2][1] = tmp[0][0][1];

			}
			if(angle == 3)
			{
				m[0][0][0] = tmp[0][2][0];
				m[0][0][1] = tmp[0][2][1];

				m[1][0][0] = tmp[0][1][0];
				m[1][0][1] = tmp[0][1][1];

				m[2][0][0] = tmp[0][0][0];
				m[2][0][1] = tmp[0][0][1];

				m[0][1][0] = tmp[1][2][0];
				m[0][1][1] = tmp[1][2][1];

				m[1][1][0] = tmp[1][1][0];
				m[1][1][1] = tmp[1][1][1];

				m[2][1][0] = tmp[1][0][0];
				m[2][1][1] = tmp[1][0][1];

				m[0][2][0] = tmp[2][2][0];
				m[0][2][1] = tmp[2][2][1];

				m[1][2][0] = tmp[2][1][0];
				m[1][2][1] = tmp[2][1][1];

				m[2][2][0] = tmp[2][0][0];
				m[2][2][1] = tmp[2][0][1];

			}

				//////////////////////////////////////////////////////////////////////////////////
				if( lastTex !=  map->textures[map->wAnimTiles[i/2+j/2*map->Header.dwXSectorsCount*16] & 0x3ff])
				{
					glBindTexture( GL_TEXTURE_2D, map->textures[map->wAnimTiles[i/2+j/2*map->Header.dwXSectorsCount*16] & 0x3ff] );
					lastTex = map->textures[map->wAnimTiles[i/2+j/2*map->Header.dwXSectorsCount*16] & 0x3ff];
				}

				SMapObject inf = map->Header.objectArray[map->wObjectID[i/2+j/2*map->Header.dwXSectorsCount*16]];
				float colors[4];
				colors[0] = inf.fpR;
				colors[1] = inf.fpG;
				colors[2] = inf.fpB;
				colors[3] = inf.fpOpacity;
				glMaterialfv( GL_FRONT, GL_DIFFUSE , colors );
				glBegin(GL_QUADS);
				glTexCoord2f(m[0][0][0],m[0][0][1]);
				wvertexToDL( i, j );
				glTexCoord2f(m[1][0][0],m[1][0][1]);
				wvertexToDL( i+1, j );
				glTexCoord2f(m[1][1][0],m[1][1][1]);
				wvertexToDL( i+1,j+1 );
				glTexCoord2f(m[0][1][0],m[0][1][1]);
				wvertexToDL( i,j+1 );
				glTexCoord2f(m[1][0][0],m[1][0][1]);
				wvertexToDL( i+1,j );
				glTexCoord2f(m[2][0][0],m[2][0][1]);
				wvertexToDL( i+2,j );
				glTexCoord2f(m[2][1][0],m[2][1][1]);
				wvertexToDL( i+2,j+1 );
				glTexCoord2f(m[1][1][0],m[1][1][1]);
				wvertexToDL( i+1,j+1 );
				glTexCoord2f(m[0][1][0],m[0][1][1]);
				wvertexToDL( i,j+1 );
				glTexCoord2f(m[1][1][0],m[1][1][1]);
				wvertexToDL( i+1,j+1 );
				glTexCoord2f(m[1][2][0],m[1][2][1]);
				wvertexToDL( i+1,j+2 );
				glTexCoord2f(m[0][2][0],m[0][2][1]);
				wvertexToDL( i,j+2 );
				glTexCoord2f(m[1][1][0],m[1][1][1]);
				wvertexToDL( i+1,j+1 );
				glTexCoord2f(m[2][1][0],m[2][1][1]);
				wvertexToDL( i+2,j+1 );
				glTexCoord2f(m[2][2][0],m[2][2][1]);
				wvertexToDL( i+2,j+2 );
				glTexCoord2f(m[1][2][0],m[1][2][1]);
				wvertexToDL( i+1,j+2 );
				glEnd();
			}
		}
		glEndList();

/////////////////////////////////////////////////////
	i = uBase_x;
	j = uBase_y;
	min=max=map->HeightMap[ i+map->iMapSizeX*j ].usZ;

	for( i=uBase_x;i<=uBase_x+uWidth;i++ )
	for( j=uBase_y;j<=uBase_y+uHeight;j++ )
	{
		if( min > map->HeightMap[ i+map->iMapSizeX*j ].usZ ) min = map->HeightMap[ i+map->iMapSizeX*j ].usZ;
		if( max < map->HeightMap[ i+map->iMapSizeX*j ].usZ ) max = map->HeightMap[ i+map->iMapSizeX*j ].usZ;
		if( map->wObjectID[i/2+j/2*map->Header.dwXSectorsCount*16] != 0xffff )
		{
			if( min > map->WaterMap [ i+map->iMapSizeX*j ].usZ ) min = map->WaterMap [ i+map->iMapSizeX*j ].usZ;
			if( max < map->WaterMap [ i+map->iMapSizeX*j ].usZ ) max = map->WaterMap [ i+map->iMapSizeX*j ].usZ;
		}
	}
	node->setPoints( (float)uBase_x,(float)uBase_y,min,(float)uBase_x+uWidth,(float)uBase_y+uHeight,max );
	}
}

void CQuadTree::drawNode_terrain(CQtrNode * node )
{
	if( isBBoxVisible(node) )
	if( node->childs != NULL )
	{
			drawNode_terrain( node->childs + 0 );
			drawNode_terrain( node->childs + 1 );
			drawNode_terrain( node->childs + 2 );
			drawNode_terrain( node->childs + 3 );
	}
	else
	{
//		count++;
		glCallList(  node->uTerrainList );
//		glFlush();
	}

}

void CQuadTree::drawNode_anim_tiles(CQtrNode * node )
{
	if( isBBoxVisible(node) )
	if( node->childs != NULL )
	{
			drawNode_anim_tiles( node->childs + 0 );
			drawNode_anim_tiles( node->childs + 1 );
			drawNode_anim_tiles( node->childs + 2 );
			drawNode_anim_tiles( node->childs + 3 );
	}
	else
	{
		glCallList( node->uAnimTilesList );
//		glFlush();
	}

}

void CQuadTree::display( bool showLand,bool showWater,bool showObjects )
{
	UpdateFrustum();
	if( showLand )
	drawNode_terrain( root );
	if( showWater )
	{
		glEnable( GL_BLEND );
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			drawNode_anim_tiles( root );
		glDisable( GL_BLEND ); 
	}
}

void CQuadTree::UpdateFrustum()
{
float t;

glPushMatrix ();
glGetFloatv(GL_MODELVIEW_MATRIX, view);
glGetFloatv(GL_PROJECTION_MATRIX, proj);
glLoadMatrixf (proj);
glMultMatrixf (view);
glGetFloatv(GL_MODELVIEW_MATRIX, clip);
glPopMatrix ();

frustum[0].normal[0] = clip[ 3] - clip[ 0];
frustum[0].normal[1] = clip[ 7] - clip[ 4];
frustum[0].normal[2] = clip[11] - clip[ 8];
frustum[0].dist = clip[15] - clip[12];

t = 1/frustum[0].normal.length();
frustum[0].normal[0] *= t;
frustum[0].normal[1] *= t;
frustum[0].normal[2] *= t;
frustum[0].dist *= t;

frustum[1].normal[0] = clip[ 3] + clip[ 0];
frustum[1].normal[1] = clip[ 7] + clip[ 4];
frustum[1].normal[2] = clip[11] + clip[ 8];
frustum[1].dist = clip[15] + clip[12];

t = 1/frustum[1].normal.length();
frustum[1].normal[0] *= t;
frustum[1].normal[1] *= t;
frustum[1].normal[2] *= t;
frustum[1].dist *= t;

frustum[2].normal[0] = clip[ 3] + clip[ 1];
frustum[2].normal[1] = clip[ 7] + clip[ 5];
frustum[2].normal[2] = clip[11] + clip[ 9];
frustum[2].dist = clip[15] + clip[13];

t = 1/frustum[2].normal.length();
frustum[2].normal[0] *= t;
frustum[2].normal[1] *= t;
frustum[2].normal[2] *= t;
frustum[2].dist *= t;

frustum[3].normal[0] = clip[ 3] - clip[ 1];
frustum[3].normal[1] = clip[ 7] - clip[ 5];
frustum[3].normal[2] = clip[11] - clip[ 9];
frustum[3].dist = clip[15] - clip[13];

t = 1/frustum[3].normal.length();
frustum[3].normal[0] *= t;
frustum[3].normal[1] *= t;
frustum[3].normal[2] *= t;
frustum[3].dist *= t;

frustum[4].normal[0] = clip[ 3] - clip[ 2];
frustum[4].normal[1] = clip[ 7] - clip[ 6];
frustum[4].normal[2] = clip[11] - clip[10];
frustum[4].dist = clip[15] - clip[14];

t = 1/frustum[4].normal.length();
frustum[4].normal[0] *= t;
frustum[4].normal[1] *= t;
frustum[4].normal[2] *= t;
frustum[4].dist *= t;

frustum[5].normal[0] = clip[ 3] + clip[ 2];
frustum[5].normal[1] = clip[ 7] + clip[ 6];
frustum[5].normal[2] = clip[11] + clip[10];
frustum[5].dist = clip[15] + clip[14];

t = 1/frustum[5].normal.length();
frustum[5].normal[0] *= t;
frustum[5].normal[1] *= t;
frustum[5].normal[2] *= t;
frustum[5].dist *= t;
}

bool CQuadTree::isBBoxVisible(CBountingVolume *node)
{
	int i,j;
	for (i=0 ; i<6 ; i++) 
	{
	for (j=0 ; j<8 ; j++) 
	{
		if (Dot3(frustum[i].normal, node->points[j]) + frustum[i].dist > 0.0)
		break;
	}

	if (j == 8)	return false;

}
return true;

}

void CQuadTree::displayBVolume(CQtrNode * node )
{
	if( isBBoxVisible(node) )
	if( node->childs != NULL  )
	{
			displayBVolume( node->childs + 0 );
			displayBVolume( node->childs + 1 );
			displayBVolume( node->childs + 2 );
			displayBVolume( node->childs + 3 );
	}
	else
	{
		if(node->color[1] == 0)
		{
//			glColor3ubv( &node->color[0] );
//			glCallList( node->uBV_list );
			glFlush();
		}
	}
}

void CQuadTree::changeLandscape( unsigned int x, unsigned int y,unsigned int z )
{
	changeNodeBBox( root,(float)x,(float)y,(float)z );
}

void CQuadTree::changeNodeBBox( CQtrNode* node,float x, float y,float z )
{
	if( (x >= node->points[0].x) && (x <= node->points[6].x) &&
		(y >= node->points[0].y) && (y <= node->points[6].y)  )
	{
		node->changed = true;
		if( z > node->points[6].z )
		{
			node->points[4].z = node->points[5].z = node->points[6].z = node->points[7].z = z;
		}
			if( node->childs != NULL )
			{
				changeNodeBBox( node->childs + 0,x,y,z );
				changeNodeBBox( node->childs + 1,x,y,z );
				changeNodeBBox( node->childs + 2,x,y,z );
				changeNodeBBox( node->childs + 3,x,y,z );
			}
	}
}

void CQuadTree::rebuildChangedNodes()
{
	updateDisplayLists( root );
}

void CQuadTree::updateDisplayLists( CQtrNode* node )
{
	if( node->changed)
	{
			if( node->childs != NULL )
			{
				updateDisplayLists( node->childs + 0 );
				updateDisplayLists( node->childs + 1 );
				updateDisplayLists( node->childs + 2 );
				updateDisplayLists( node->childs + 3 );
			}
			else
			{
						buildNode(	node,
							(int)node->points[0].x,
							(int)node->points[0].y,
							(int)(node->points[6].x - node->points[0].x),
							(int)(node->points[6].y - node->points[0].y));
			}
			node->changed = false;
	}
}

void CQuadTree::displayWireFrame()
{
	drawNode_terrainWireFrame( root );
	drawNode_anim_tilesWireFrame( root );
}

void CQuadTree::drawNode_terrainWireFrame(CQtrNode* node )
{
	if( isBBoxVisible(node) )
	if( node->childs != NULL )
	{
			drawNode_terrainWireFrame( node->childs + 0 );
			drawNode_terrainWireFrame( node->childs + 1 );
			drawNode_terrainWireFrame( node->childs + 2 );
			drawNode_terrainWireFrame( node->childs + 3 );
	}
	else
	{
		glCallList(  node->uTerrainList );
	}

}

void CQuadTree::drawNode_anim_tilesWireFrame( CQtrNode* node )
{
	if( isBBoxVisible(node) )
	if( node->childs != NULL )
	{
			drawNode_anim_tilesWireFrame( node->childs + 0 );
			drawNode_anim_tilesWireFrame( node->childs + 1 );
			drawNode_anim_tilesWireFrame( node->childs + 2 );
			drawNode_anim_tilesWireFrame( node->childs + 3 );
	}
	else
	{
		glCallList( node->uAnimTilesList );
	}


}

void CQuadTree::displayNormals()
{
	displayNodeNromals( root );
}

void CQuadTree::displayNodeNromals( CQtrNode *node )
{
	if( isBBoxVisible(node) )
	if( node->childs != NULL )
	{
			displayNodeNromals( node->childs + 0 );
			displayNodeNromals( node->childs + 1 );
			displayNodeNromals( node->childs + 2 );
			displayNodeNromals( node->childs + 3 );
	}
	else
	{
		glBegin(GL_LINES);
		for( int j=(int)node->points[0].y;j<(int)node->points[2].y;j++ )
		for( int i=(int)node->points[0].x;i<(int)node->points[2].x;i++ )
		{
			vec3	n;
			
			GetNormal( map->HeightMap[i+map->iMapSizeX*j].uNormal, n.x, n.y, n.z  );
			
			glVertex3f((float)i,(float)j,map->getHMapZ(i,j)*map->getvZoom()/65536.0f);
			glVertex3f((float)i+n.x,(float)j+n.y,map->getvZoom()/65536.0f*map->getHMapZ(i,j)+n.z);
		}
		glEnd();
	}
}

void CQuadTree::vertexToDL( int x, int y )
{
	static	vec3	v;
	GetNormal( map->HeightMap[x+(map->Header.dwXSectorsCount*32+1)*y].uNormal, v.x,v.y,v.z);
	glNormal3f( v.x,v.y,v.z );
	glVertex3f( (float)x , (float)y , (float)map->HeightMap[x+  (map->Header.dwXSectorsCount*32+1)*y].usZ*(float)map->getvZoom()/65535.0f );
}

void CQuadTree::wvertexToDL( int x, int y )
{
	static	vec3	v;
	GetNormal( map->WaterMap[x+(map->Header.dwXSectorsCount*32+1)*y].uNormal, v.x,v.y,v.z);
	glNormal3f( v.x,v.y,v.z );
	glVertex3f( (float)x , (float)y , (float)map->WaterMap[x+  (map->Header.dwXSectorsCount*32+1)*y].usZ*(float)map->getvZoom()/65535.0f );
}

void CQuadTree::setNodeChanged( CQtrNode* node, int x, int y )
{
	if( (x >= (int)node->points[0].x) && (x <= (int)node->points[6].x) &&
		(y >= (int)node->points[0].y) && (y <= (int)node->points[6].y)  )
	{
		node->changed = true;
			if( node->childs != NULL )
			{
				setNodeChanged( node->childs + 0,x,y );
				setNodeChanged( node->childs + 1,x,y );
				setNodeChanged( node->childs + 2,x,y );
				setNodeChanged( node->childs + 3,x,y );
			}
	}

}
